// C source for perlc v.2.0
// Compile Perl script to statically linked relocatable executable and
// copy its modules in a relative-path 'lib' folder.
// Authors:
// David Ljung Madison <marginalhacks.com/Hacks/perlc/>
// Dimitar D. Mitov <github.com/ddmitov>
// License: http://MarginalHacks.com/License/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "aes256.h"

void dump(uint8_t *buffer, size_t size);

int main (int argc, char *argv[])
{
  if (argv[1] && argv[2]) {
    FILE *file = fopen(argv[1], "rt");
    fseek(file, 0, SEEK_END);
    long fsize = ftell(file);
    fseek(file, 0, SEEK_SET);

    char *file_contents = malloc(fsize + 1);
    fread(file_contents, fsize, 1, file);

    fclose(file);

    char disk_serial[48];

    if (strcmp("current", argv[2]) == 0) {
      char *cwd = getcwd(0, 0);
      char device[10];

      FILE *df_process;

      char *df_command = "df ";
      char *get_device_filter_commands = " | grep dev | cut -d' ' -f1";

      char get_device_command[strlen(df_command) +
                              strlen(cwd) +
                              strlen(get_device_filter_commands)];
      strcpy(get_device_command, df_command);
      strcat(get_device_command, cwd);
      strcat(get_device_command, get_device_filter_commands);

      if ((df_process = popen(get_device_command, "r")) == NULL) {
        printf("Error starting 'df'!\n");
        return -1;
      }

      if (fgets(device, 10, df_process) != NULL) {
        FILE *udevadm_process;

        char *udevadm_command = "udevadm info --query=all --name=";
        char *udevadm_filter_commands = " | grep ID_SERIAL= | cut -d'=' -f2";

        char get_disk_number_command[strlen(udevadm_command) +
                                    strlen(device) +
                                    strlen(udevadm_filter_commands)];
        strcpy(get_disk_number_command, udevadm_command);
        strcat(get_disk_number_command, device);
        strcat(get_disk_number_command, udevadm_filter_commands);

        if ((udevadm_process = popen(get_disk_number_command, "r")) == NULL) {
          printf("Error starting 'udevadm'!\n");
          return -1;
        }

        fgets(disk_serial, 48, udevadm_process);

        if(pclose(udevadm_process)) {
          printf("'udevadm' command was not found or exited with error!\n");
          return -1;
        }
      }

      if(pclose(df_process)) {
        printf("'df' command was not found or exited with error!\n");
        return -1;
      }
    }

    if (strcmp("current", argv[2]) != 0) {
      sscanf(argv[2], "%s", disk_serial);
    }

    rfc3686_blk ctr = {
      {0x00, 0x1C, 0xC5, 0xB7}, /* nonce */
      {0x51, 0xA5, 0x1D, 0x70, 0xA1, 0xC1, 0x11, 0x48}, /* IV */
      {0x00, 0x00, 0x00, 0x01} /* counter */
    };

    aes256_context ctx;

    uint8_t *encryption_buffer = (uint8_t *)(file_contents);
    size_t buffer_length = strlen(file_contents);

    aes256_init(&ctx, (uint8_t *)(&disk_serial));
    aes256_setCtrBlk(&ctx, &ctr);
    aes256_encrypt_ctr(&ctx, encryption_buffer, buffer_length);
    aes256_done(&ctx);

    dump(encryption_buffer, buffer_length);
  }
}

void dump(uint8_t *buffer, size_t size)
{
  size_t index;
  for (index = 0; index < size; index++) {
    printf("0x%02x%s", buffer[index],
      ((index % 16 == 15) && (index < size - 1)) ? ",\n" : ", ");
  }
}
