// C source for perlc v.2.0
// Compile Perl script to statically linked relocatable executable and
// copy its modules in a relative-path 'lib' folder.
// Authors:
// David Ljung Madison <marginalhacks.com/Hacks/perlc/>
// Dimitar D. Mitov <github.com/ddmitov>
// License: http://MarginalHacks.com/License/

/* Compile command (wrap for better view):
//COMPILE-COMMAND//
*/

#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <unistd.h>
//DYNALIB//
#include "aes256.h"

//SCRIPT//

#ifdef _WIN32
  char *path_separator = "\\";
#else
  char *path_separator = "/";
#endif

int main(int argc, char **argv, char **env) {
  // Get the current working directory:
  char *cwd = getcwd(0, 0);

  // Get the serial number of the current disk drive:
  char device[10];
  char disk_serial[48];

  FILE *df_process;

  char *df_command = "df ";
  char *get_device_filter_commands = " | grep dev | cut -d' ' -f1";

  char get_device_command[strlen(df_command) +
                          strlen(cwd) +
                          strlen(get_device_filter_commands)];
  strcpy(get_device_command, df_command);
  strcat(get_device_command, cwd);
  strcat(get_device_command, get_device_filter_commands);

  if ((df_process = popen(get_device_command, "r")) == NULL) {
    printf("Error starting 'df'!\n");
    return -1;
  }

  if (fgets(device, 10, df_process) != NULL) {
    FILE *udevadm_process;

    char *udevadm_command = "udevadm info --query=all --name=";
    char *udevadm_filter_commands = " | grep ID_SERIAL= | cut -d'=' -f2";

    char get_disk_number_command[strlen(udevadm_command) +
                                strlen(device) +
                                strlen(udevadm_filter_commands)];
    strcpy(get_disk_number_command, udevadm_command);
    strcat(get_disk_number_command, device);
    strcat(get_disk_number_command, udevadm_filter_commands);

    if ((udevadm_process = popen(get_disk_number_command, "r")) == NULL) {
      printf("Error starting 'udevadm'!\n");
      return -1;
    }

    fgets(disk_serial, 48, udevadm_process);

    if(pclose(udevadm_process)) {
      printf("'udevadm' command was not found or exited with error status!\n");
      return -1;
    }
  }

  if(pclose(df_process)) {
    printf("'df' command was not found or exited with error status!\n");
    return -1;
  }

  // Decryption code:
  rfc3686_blk ctr = {
    {0x00, 0x1C, 0xC5, 0xB7}, /* nonce */
    {0x51, 0xA5, 0x1D, 0x70, 0xA1, 0xC1, 0x11, 0x48}, /* IV */
    {0x00, 0x00, 0x00, 0x01} /* counter */
  };

  aes256_context ctx;

  uint8_t *decryption_buffer = (uint8_t *)(&script);
  aes256_init(&ctx, (uint8_t *)(&disk_serial));
  aes256_setCtrBlk(&ctx, &ctr);
  aes256_decrypt_ctr(&ctx, decryption_buffer, sizeof(script));
  aes256_done(&ctx);

  // Compose the full path to the 'lib' folder:
  char *perl5lib =
    malloc(1 + strlen(cwd) + strlen(path_separator) + strlen("lib"));
  strcpy(perl5lib, cwd);
  strcat(perl5lib, path_separator);
  strcat(perl5lib, "lib");

  // Compose the Perl code:
  char *perl_wrapper_code_01 = "BEGIN {$0=$^X; splice @INC, 0; push @INC, \"";
  char *perl_wrapper_code_02 = "\";}\n eval '";
  char *perl_wrapper_code_03 =
    "'; if ($@ and $@ =~ 'Unrecognized') {print 'Unauthorized access!\n';}";

  char perl_code[strlen(perl_wrapper_code_01) +
                strlen(perl5lib) +
                strlen(perl_wrapper_code_02) +
                strlen((char *)(decryption_buffer)) +
                strlen(perl_wrapper_code_03)];
  strcpy(perl_code, perl_wrapper_code_01);
  strcat(perl_code, perl5lib);
  strcat(perl_code, perl_wrapper_code_02);
  strcat(perl_code, (char *)(decryption_buffer));
  strcat(perl_code, perl_wrapper_code_03);

  // Perl-specific command-line arguments,
  // embedded script is executed as one-liner:
  char **perl_argv = malloc(1);
  perl_argv[0] = malloc(1 + strlen(""));
  strcpy(perl_argv[0], "");
  perl_argv[1] = malloc(1 + strlen("-e"));
  strcpy(perl_argv[1], "-e");
  perl_argv[2] = malloc(1 + strlen(perl_code));
  strcpy(perl_argv[2], perl_code);
  perl_argv[3] = malloc(1 + strlen("--"));
  strcpy(perl_argv[3], "--");

  // Combine Perl-specific command-line arguments with
  // the command-line arguments supplied to the executable:
  int perl_argc;
  for(perl_argc = 4; perl_argc < 3 + argc; perl_argc++) {
    // argv[0] - the name of the executable - is omitted from
    // the command-line arguments supplied to the embedded script.
    perl_argv[perl_argc] = malloc(strlen(argv[perl_argc - 3]));
    strcpy(perl_argv[perl_argc], argv[perl_argc - 3]);
  }

  // Initialize Perl interpreter and execute the embedded script:
  static PerlInterpreter *interpreter;
  interpreter = perl_alloc();
  perl_construct(interpreter);

  perl_parse(interpreter, xs_init, perl_argc, perl_argv, (char **) NULL);
  PL_exit_flags |= PERL_EXIT_DESTRUCT_END;
  perl_run(interpreter);

  perl_destruct(interpreter);
  perl_free(interpreter);
  fflush(stdout);
  return 0;
}
