// C source for perlc v.2.0
// Compile Perl script to statically linked relocatable executable and
// copy its modules in a relative-path 'lib' folder.
// Authors:
// David Ljung Madison <marginalhacks.com/Hacks/perlc/>
// Dimitar D. Mitov <github.com/ddmitov>
// License: http://MarginalHacks.com/License/

/* Compile command (wrap for better view):
//COMPILE-COMMAND//
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <unistd.h>

//DYNALIB//

#include "b64.h"

//SCRIPT//

#ifdef _WIN32
  char *path_separator = "\\";
#else
  char *path_separator = "/";
#endif

int main(int argc, char **argv, char **env) {
  // Get the current working directory:
  char *cwd = getcwd(0, 0);

  // Compose the full path to the 'lib' folder:
  char *perl5lib =
    malloc(1 + strlen(cwd) + strlen(path_separator) + strlen("lib"));
  strcpy(perl5lib, cwd);
  strcat(perl5lib, path_separator);
  strcat(perl5lib, "lib");

  // Decode the embedded Perl script from Base64 encoding:
  unsigned char *decoded_script = b64_decode(script, strlen(script));

  // Perl code composition:
  char *perl_wrapper_code_01 = "BEGIN {$0=$^X; splice @INC, 0; push @INC, \"";
  char *perl_wrapper_code_02 = "\";}";

  char perl_code[strlen(perl_wrapper_code_01) +
                strlen(perl5lib) +
                strlen(perl_wrapper_code_02) +
                strlen((char *)(decoded_script))];
  strcpy(perl_code, perl_wrapper_code_01);
  strcat(perl_code, perl5lib);
  strcat(perl_code, perl_wrapper_code_02);
  strcat(perl_code, (char *)(decoded_script));

  // Perl-specific command-line arguments,
  // embedded script is executed as one-liner:
  char **perl_argv = malloc(1);
  perl_argv[0] = malloc(1 + strlen(""));
  strcpy(perl_argv[0], "");
  perl_argv[1] = malloc(1 + strlen("-e"));
  strcpy(perl_argv[1], "-e");
  perl_argv[2] = malloc(1 + strlen(perl_code));
  strcpy(perl_argv[2], perl_code);
  perl_argv[3] = malloc(1 + strlen("--"));
  strcpy(perl_argv[3], "--");

  // Combine Perl-specific command-line arguments with
  // the command-line arguments supplied to the executable:
  int perl_argc;
  for(perl_argc = 4; perl_argc < 3 + argc; perl_argc++) {
    // argv[0] - the name of the executable - is omitted from
    // the command-line arguments supplied to the embedded script.
    perl_argv[perl_argc] = malloc(strlen(argv[perl_argc - 3]));
    strcpy(perl_argv[perl_argc], argv[perl_argc - 3]);
  }

  // Initialize Perl interpreter and execute the embedded script:
  static PerlInterpreter *interpreter;
  interpreter = perl_alloc();
  perl_construct(interpreter);
  perl_parse(interpreter, xs_init, perl_argc, perl_argv, (char **)NULL);
  perl_run(interpreter);

  perl_destruct(interpreter);
  perl_free(interpreter);
  fflush(stdout);

  return 0;
}
